#coding=utf-8
'''
android手机相关action设定
Created on 2017年6月13日

@author: Administrator
'''

class SettingAction(object):
    
    ACTION_SETTINGS = "android.settings.SETTINGS" #设置界面
    ACTION_APN_SETTINGS = "android.settings.APN_SETTINGS"
    ACTION_LOCATION_SOURCE_SETTINGS ="android.settings.LOCATION_SOURCE_SETTINGS"
    ACTION_USER_SETTINGS ="android.settings.USER_SETTINGS" # system
    ACTION_WIRELESS_SETTINGS ="android.settings.WIRELESS_SETTINGS"
    ACTION_TETHER_PROVISIONING ="android.settings.TETHER_PROVISIONING_UI"  #hide
    ACTION_AIRPLANE_MODE_SETTINGS ="android.settings.AIRPLANE_MODE_SETTINGS"
    ACTION_VOICE_CONTROL_AIRPLANE_MODE ="android.settings.VOICE_CONTROL_AIRPLANE_MODE"
    ACTION_ACCESSIBILITY_SETTINGS = "android.settings.ACCESSIBILITY_SETTINGS"
    ACTION_USAGE_ACCESS_SETTINGS = "android.settings.USAGE_ACCESS_SETTINGS"
    INTENT_CATEGORY_USAGE_ACCESS_CONFIG ="android.intent.category.USAGE_ACCESS_CONFIG"
    ACTION_SECURITY_SETTINGS ="android.settings.SECURITY_SETTINGS"
    ACTION_TRUSTED_CREDENTIALS_USER ="com.android.settings.TRUSTED_CREDENTIALS_USER"
    ACTION_MONITORING_CERT_INFO ="com.android.settings.MONITORING_CERT_INFO"
    ACTION_PRIVACY_SETTINGS ="android.settings.PRIVACY_SETTINGS"
    ACTION_VPN_SETTINGS ="android.settings.VPN_SETTINGS"
    ACTION_WIFI_SETTINGS ="android.settings.WIFI_SETTINGS"
    ACTION_WIFI_IP_SETTINGS ="android.settings.WIFI_IP_SETTINGS"
    ACTION_BLUETOOTH_SETTINGS ="android.settings.BLUETOOTH_SETTINGS"
    ACTION_CAST_SETTINGS ="android.settings.CAST_SETTINGS"
    ACTION_DATE_SETTINGS ="android.settings.DATE_SETTINGS"
    ACTION_SOUND_SETTINGS ="android.settings.SOUND_SETTINGS"
    ACTION_DISPLAY_SETTINGS ="android.settings.DISPLAY_SETTINGS"
    ACTION_NIGHT_DISPLAY_SETTINGS ="android.settings.NIGHT_DISPLAY_SETTINGS"
    ACTION_LOCALE_SETTINGS ="android.settings.LOCALE_SETTINGS"
    ACTION_VOICE_INPUT_SETTINGS ="android.settings.VOICE_INPUT_SETTINGS"
    ACTION_INPUT_METHOD_SETTINGS ="android.settings.INPUT_METHOD_SETTINGS"
    ACTION_INPUT_METHOD_SUBTYPE_SETTINGS ="android.settings.INPUT_METHOD_SUBTYPE_SETTINGS"
    ACTION_SHOW_INPUT_METHOD_PICKER ="android.settings.SHOW_INPUT_METHOD_PICKER"
    ACTION_USER_DICTIONARY_SETTINGS ="android.settings.USER_DICTIONARY_SETTINGS"
    ACTION_HARD_KEYBOARD_SETTINGS ="android.settings.HARD_KEYBOARD_SETTINGS"
    ACTION_USER_DICTIONARY_INSERT ="com.android.settings.USER_DICTIONARY_INSERT"
    ACTION_APPLICATION_SETTINGS ="android.settings.APPLICATION_SETTINGS"
    ACTION_APPLICATION_DEVELOPMENT_SETTINGS ="android.settings.APPLICATION_DEVELOPMENT_SETTINGS"
    ACTION_QUICK_LAUNCH_SETTINGS ="android.settings.QUICK_LAUNCH_SETTINGS"
    ACTION_MANAGE_APPLICATIONS_SETTINGS ="android.settings.MANAGE_APPLICATIONS_SETTINGS"
    ACTION_MANAGE_ALL_APPLICATIONS_SETTINGS ="android.settings.MANAGE_ALL_APPLICATIONS_SETTINGS"
    ACTION_MANAGE_OVERLAY_PERMISSION ="android.settings.action.MANAGE_OVERLAY_PERMISSION"
    ACTION_MANAGE_WRITE_SETTINGS ="android.settings.action.MANAGE_WRITE_SETTINGS"
    ACTION_APPLICATION_DETAILS_SETTINGS ="android.settings.APPLICATION_DETAILS_SETTINGS"
    ACTION_IGNORE_BATTERY_OPTIMIZATION_SETTINGS ="android.settings.IGNORE_BATTERY_OPTIMIZATION_SETTINGS"
    ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS ="android.settings.REQUEST_IGNORE_BATTERY_OPTIMIZATIONS"
    ACTION_IGNORE_BACKGROUND_DATA_RESTRICTIONS_SETTINGS ="android.settings.IGNORE_BACKGROUND_DATA_RESTRICTIONS_SETTINGS"
    ACTION_SYSTEM_UPDATE_SETTINGS ="android.settings.SYSTEM_UPDATE_SETTINGS"
    ACTION_SYNC_SETTINGS ="android.settings.SYNC_SETTINGS"
    ACTION_ADD_ACCOUNT ="android.settings.ADD_ACCOUNT_SETTINGS"
    ACTION_NETWORK_OPERATOR_SETTINGS ="android.settings.NETWORK_OPERATOR_SETTINGS"
    ACTION_DATA_ROAMING_SETTINGS ="android.settings.DATA_ROAMING_SETTINGS"
    ACTION_INTERNAL_STORAGE_SETTINGS ="android.settings.INTERNAL_STORAGE_SETTINGS"
    ACTION_MEMORY_CARD_SETTINGS ="android.settings.MEMORY_CARD_SETTINGS"
    ACTION_SEARCH_SETTINGS ="android.search.action.SEARCH_SETTINGS"
    ACTION_DEVICE_INFO_SETTINGS ="android.settings.DEVICE_INFO_SETTINGS"
    ACTION_NFC_SETTINGS = "android.settings.NFC_SETTINGS"
    ACTION_NFCSHARING_SETTINGS ="android.settings.NFCSHARING_SETTINGS"
    ACTION_NFC_PAYMENT_SETTINGS ="android.settings.NFC_PAYMENT_SETTINGS"
    ACTION_DREAM_SETTINGS = "android.settings.DREAM_SETTINGS"
    ACTION_NOTIFICATION_LISTENER_SETTINGS = "android.settings.ACTION_NOTIFICATION_LISTENER_SETTINGS"
    ACTION_NOTIFICATION_POLICY_ACCESS_SETTINGS = "android.settings.NOTIFICATION_POLICY_ACCESS_SETTINGS"
    ACTION_CONDITION_PROVIDER_SETTINGS = "android.settings.ACTION_CONDITION_PROVIDER_SETTINGS"
    ACTION_CAPTIONING_SETTINGS = "android.settings.CAPTIONING_SETTINGS"
    ACTION_PRINT_SETTINGS ="android.settings.ACTION_PRINT_SETTINGS"
    ACTION_ZEN_MODE_SETTINGS = "android.settings.ZEN_MODE_SETTINGS"
    ACTION_ZEN_MODE_PRIORITY_SETTINGS = "android.settings.ZEN_MODE_PRIORITY_SETTINGS"
    ACTION_ZEN_MODE_AUTOMATION_SETTINGS = "android.settings.ZEN_MODE_AUTOMATION_SETTINGS"
    ACTION_VOICE_CONTROL_DO_NOT_DISTURB_MODE = "android.settings.VOICE_CONTROL_DO_NOT_DISTURB_MODE"
    ACTION_ZEN_MODE_SCHEDULE_RULE_SETTINGS = "android.settings.ZEN_MODE_SCHEDULE_RULE_SETTINGS"
    ACTION_ZEN_MODE_EVENT_RULE_SETTINGS = "android.settings.ZEN_MODE_EVENT_RULE_SETTINGS"
    ACTION_ZEN_MODE_EXTERNAL_RULE_SETTINGS = "android.settings.ZEN_MODE_EXTERNAL_RULE_SETTINGS" # hide
    ACTION_SHOW_REGULATORY_INFO = "android.settings.SHOW_REGULATORY_INFO"
    DEVICE_NAME_SETTINGS = "android.settings.DEVICE_NAME" #hide
    ACTION_PAIRING_SETTINGS = "android.settings.PAIRING_SETTINGS" #hide
    ACTION_BATTERY_SAVER_SETTINGS = "android.settings.BATTERY_SAVER_SETTINGS"
    ACTION_VOICE_CONTROL_BATTERY_SAVER_MODE = "android.settings.VOICE_CONTROL_BATTERY_SAVER_MODE"
    ACTION_HOME_SETTINGS = "android.settings.HOME_SETTINGS"
    ACTION_MANAGE_DEFAULT_APPS_SETTINGS = "android.settings.MANAGE_DEFAULT_APPS_SETTINGS"
    ACTION_NOTIFICATION_SETTINGS = "android.settings.NOTIFICATION_SETTINGS" #hide
    ACTION_APP_NOTIFICATION_SETTINGS = "android.settings.APP_NOTIFICATION_SETTINGS" #hide
    ACTION_APP_NOTIFICATION_REDACTION = "android.settings.ACTION_APP_NOTIFICATION_REDACTION" #hide
    ACTION_SHOW_ADMIN_SUPPORT_DETAILS = "android.settings.SHOW_ADMIN_SUPPORT_DETAILS" #hide
    ACTION_SHOW_REMOTE_BUGREPORT_DIALOG = "android.settings.SHOW_REMOTE_BUGREPORT_DIALOG" #hide
    ACTION_VR_LISTENER_SETTINGS = "android.settings.VR_LISTENER_SETTINGS"
    ACTION_STORAGE_MANAGER_SETTINGS = "android.settings.STORAGE_MANAGER_SETTINGS"
    ACTION_WEBVIEW_SETTINGS = "android.settings.WEBVIEW_SETTINGS"
    

        