# coding=utf-8
'''
Created on 2016年4月15日
@author: fengbo
'''

class ErrorCode(object):
    '''
    Error Code.错误码
    '''
    DEVICE_EXECEPTION = 9
    POWER_MONITOR_STARTUP = 18
    COMMAND_TIME_OUT = 21
    SOCKET_CONNECT_FAIL = 10
    SUCCESS = 0
    FAIL = 1 